@@ -35,7 +35,7 @@ def rectangles_collide(x1, y1, w1, h1, x2, y2, w2, h2):
     """
     Return whether or not two rectangles collide.
 
-    Arguments:
+    Parameters:
 
     - ``x1`` -- The horizontal position of the first rectangle.
     - ``y1`` -- The vertical position of the first rectangle.
@@ -53,7 +53,7 @@ def masks_collide(x1, y1, mask1, x2, y2, mask2):
     """
     Return whether or not two masks collide.
 
-    Arguments:
+    Parameters:
 
     - ``x1`` -- The horizontal position of the first mask.
     - ``y1`` -- The vertical position of the first mask.
@@ -93,7 +93,7 @@ def rectangle(x, y, w, h, other=None):
     """
     Return a list of objects colliding with a rectangle.
 
-    Arguments:
+    Parameters:
 
     - ``x`` -- The horizontal position of the rectangle.
     - ``y`` -- The vertical position of the rectangle.
@@ -131,7 +131,7 @@ def ellipse(x, y, w, h, other=None):
     """
     Return a list of objects colliding with an ellipse.
 
-    Arguments:
+    Parameters:
 
     - ``x`` -- The horizontal position of the imaginary rectangle
       containing the ellipse.
@@ -155,8 +155,8 @@ def ellipse(x, y, w, h, other=None):
         a = len(mask) / 2
         b = len(mask[0]) / 2 if mask else 0
 
-        for i in range(len(mask)):
-            for j in range(len(mask[i])):
+        for i, column in enumerate(mask):
+            for j, row in enumerate(column):
                 if ((i - a) / a) ** 2 + ((j - b) / b) ** 2 <= 1:
                     mask[i][j] = True
 
@@ -174,7 +174,7 @@ def circle(x, y, radius, other=None):
     """
     Return a list of objects colliding with a circle.
 
-    Arguments:
+    Parameters:
 
     - ``x`` -- The horizontal position of the center of the circle.
     - ``y`` -- The vertical position of the center of the circle.
@@ -195,8 +195,8 @@ def circle(x, y, radius, other=None):
         mask = [[False for j in range(int(diameter))]
                 for i in range(int(diameter))]
 
-        for i in range(len(mask)):
-            for j in range(len(mask[i])):
+        for i, column in enumerate(mask):
+            for j, row in enumerate(column):
                 if (i - x) ** 2 + (j - y) ** 2 <= radius ** 2:
                     mask[i][j] = True
 
@@ -215,7 +215,7 @@ def line(x1, y1, x2, y2, other=None):
     """
     Return a list of objects colliding with a line segment.
 
-    Arguments:
+    Parameters:
 
     - ``x1`` -- The horizontal position of the first endpoint of the
       line segment.
