@@ -22,6 +22,7 @@ functions and variables.
 
 import inspect
 import math
+import os
 import random
 import time
 import warnings
@@ -60,8 +61,11 @@ _prev_hats = {}
 # Display info
 _display_info = None
 
+# Index of sound initialization
+sound_init = 0
 
-class cache(object):
+
+class cache:
 
     prune_time = 0
     _cache = {}
@@ -103,7 +107,7 @@ class cache(object):
 
 def _check_color_input(value):
     # Make sure a color value is between 0 and 255.
-    if value in range(256):
+    if 0 <= value <= 255:
         return value
     else:
         raise ValueError("Color values must be between 0 and 255.")
@@ -166,28 +170,52 @@ def _get_blend_flags(blend_mode):
 def _screen_blend(dest, source, dest_x, dest_y, alpha=False):
     dest.lock()
     source.lock()
-    for y in range(source.get_height()):
-        if 0 <= dest_y + y < dest.get_height():
-            for x in range(source.get_width()):
-                if 0 <= dest_x + x < dest.get_width():
-                    dc = dest.get_at((dest_x + x, dest_y + y))
-                    sc = source.get_at((x, y))
+    destpix = pygame.PixelArray(dest)
+    sourcepix = pygame.PixelArray(source)
+    xmin = max(0, dest_x)
+    ymin = max(0, dest_y)
+    xlen = min(source.get_width(), dest.get_width() - dest_x)
+    ylen = min(source.get_height(), dest.get_height() - dest_y)
 
-                    def blended_component(c1, c2):
-                        return int(255 - (((255 - c1) / 255) *
-                                          ((255 - c2) / 255) * 255))
+    for x in range(xmin, xlen):
+        for y in range(ymin, ylen):
+            dx = dest_x + x
+            dy = dest_y + y
+            dc = dest.unmap_rgb(destpix[dx, dy])
+            sc = source.unmap_rgb(sourcepix[x, y])
 
-                    r = blended_component(dc.r, sc.r)
-                    g = blended_component(dc.g, sc.g)
-                    b = blended_component(dc.b, sc.b)
-                    a = blended_component(dc.a, sc.a) if alpha else dc.a
+            def blended_component(c1, c2):
+                return int(255 - (((255 - c1) / 255) *
+                                  ((255 - c2) / 255) * 255))
 
-                    dest.set_at((dest_x + x, dest_y + y), pygame.Color(r, g, b, a))
+            r = blended_component(dc.r, sc.r)
+            g = blended_component(dc.g, sc.g)
+            b = blended_component(dc.b, sc.b)
+            a = blended_component(dc.a, sc.a) if alpha else dc.a
+
+            destpix[dx, dy] = dest.map_rgb((r, g, b, a))
+
+    destpix.close()
+    sourcepix.close()
     dest.unlock()
     source.unlock()
 
 
-def _set_mode():
+def _apply_shader(dest, x, y, width, height, shader):
+    # Apply a shader to surface ``dest``.  Note: caller must lock() and
+    # unlock() ``dest``.
+    pixarray = pygame.PixelArray(dest)
+    range_x = range(max(x, 0), min(x + width, dest.get_width()))
+    range_y = range(max(y, 0), min(y + height, dest.get_height()))
+    for xx in range_x:
+        for yy in range_y:
+            red, green, blue, alpha = dest.unmap_rgb(pixarray[xx, yy])
+            color = dest.map_rgb(shader(xx, yy, red, green, blue, alpha))
+            pixarray[xx, yy] = color
+    pixarray.close()
+
+
+def _set_mode(resize_only=False):
     # Set the mode of the screen based on self.width, self.height,
     # and self.fullscreen.
     global game_display_surface
@@ -208,8 +236,8 @@ def _set_mode():
     if game.fullscreen or not _display_info.wm:
         flags = pygame.FULLSCREEN | pygame.HWSURFACE | pygame.DOUBLEBUF
 
-        modes = pygame.display.list_modes(0, flags)
-        if modes == -1 or not modes:
+        modes = sge.dsp.list_fullscreen_modes()
+        if not modes:
             w = "Couldn't find out the maximum resolution! Assuming 1024x768."
             warnings.warn(w)
             modes = [(1024, 768)]
@@ -221,15 +249,23 @@ def _set_mode():
             game_window_height = int(game.height * game.scale)
             if not pygame.display.mode_ok(
                     (game_window_width, game_window_height), flags):
-                game_window_width = _display_info.current_w
-                game_window_height = _display_info.current_h
-                force_auto_scale = True
+                for mode in modes:
+                    if (mode[0] >= game_window_width
+                            and mode[1] >= game_window_height
+                            and pygame.display.mode_ok(mode, flags)):
+                        game_window_width, game_window_height = mode
+                        break
+                else:
+                    game_window_width = 0
+                    game_window_height = 0
+                    force_auto_scale = True
         else:
-            game_window_width = _display_info.current_w
-            game_window_height = _display_info.current_h
+            game_window_width = 0
+            game_window_height = 0
 
         game_window = pygame.display.set_mode(
             (game_window_width, game_window_height), flags)
+        game_window_width, game_window_height = game_window.get_size()
 
         if not game.scale or force_auto_scale:
             game_xscale = game_window_width / game.width
@@ -239,6 +275,12 @@ def _set_mode():
                 game_xscale = min(game_xscale, game_yscale)
                 game_yscale = game_xscale
 
+            if game.scale_integer:
+                if game_xscale > 1:
+                    game_xscale = math.floor(game_xscale)
+                if game_yscale > 1:
+                    game_yscale = math.floor(game_yscale)
+
         w = max(1, game_window.get_width())
         h = max(1, game_window.get_height())
         game_x = round((w - game.width*game_xscale) / 2)
@@ -260,10 +302,22 @@ def _set_mode():
                 game_xscale = min(game_xscale, game_yscale)
                 game_yscale = game_xscale
 
+            if game.scale_integer:
+                if game_xscale > 1:
+                    game_xscale = math.floor(game_xscale)
+                if game_yscale > 1:
+                    game_yscale = math.floor(game_yscale)
+
             flags |= pygame.RESIZABLE
 
-        game_window = pygame.display.set_mode(
-            (game_window_width, game_window_height), flags)
+        # If we're just resizing the window and using SDL2, don't call
+        # set_mode(), since it's unnecessary and can cause problems on
+        # GNOME, described here:
+        # https://github.com/pygame/pygame/issues/2418
+        if not resize_only or pygame.get_sdl_version()[0] < 2:
+            game_window = pygame.display.set_mode(
+                (game_window_width, game_window_height), flags)
+            game_window_width, game_window_height = game_window.get_size()
 
         w = max(1, game_window.get_width())
         h = max(1, game_window.get_height())
@@ -319,6 +373,20 @@ def _handle_music():
                 music.stop()
 
 
+def _deinit_sound():
+    # Uninitialize audio.
+    pygame.mixer.stop()
+    pygame.mixer.quit()
+
+
+def _reinit_sound():
+    # Re-initialize audio after it has been uninitialized.
+    global sound_init
+    pygame.mixer.init(sge.game.sampling_frequency, -16,
+                      2 if sge.game.stereo else 1, 512)
+    sound_init += 1
+
+
 def _get_dot_sprite(color):
     # Return a sprite for the given dot.
     i = ("dot_sprite", tuple(color))
@@ -339,7 +407,8 @@ def _get_line_sprite(x1, y1, x2, y2, color, thickness, anti_alias):
     sprite = cache.get(i)
     if sprite is None:
         sprite = sge.gfx.Sprite(None, width=w, height=h)
-        sprite.draw_line(x1, y1, x2, y2, color, thickness, anti_alias)
+        sprite.draw_line(x1, y1, x2, y2, color, thickness,
+                         anti_alias=anti_alias)
 
     cache.add(i, sprite)
     return sprite
@@ -359,8 +428,9 @@ def _get_rectangle_sprite(width, height, fill, outline, outline_thickness):
         w = width + outline_thickness
         h = height + outline_thickness
         sprite = sge.gfx.Sprite(None, width=w, height=h)
-        sprite.draw_rectangle(draw_x, draw_y, width, height, fill, outline,
-                              outline_thickness)
+        sprite.draw_rectangle(
+            draw_x, draw_y, width, height, fill=fill, outline=outline,
+            outline_thickness=outline_thickness)
 
     cache.add(i, sprite)
     return sprite
@@ -381,8 +451,9 @@ def _get_ellipse_sprite(width, height, fill, outline, outline_thickness,
         w = width + outline_thickness
         h = height + outline_thickness
         sprite = sge.gfx.Sprite(None, width=w, height=h)
-        sprite.draw_ellipse(draw_x, draw_y, width, height, fill, outline,
-                            outline_thickness)
+        sprite.draw_ellipse(
+            draw_x, draw_y, width, height, fill=fill, outline=outline,
+            outline_thickness=outline_thickness, anti_alias=anti_alias)
 
     cache.add(i, sprite)
     return sprite
@@ -399,8 +470,38 @@ def _get_circle_sprite(radius, fill, outline, outline_thickness, anti_alias):
         xy = radius + outline_thickness // 2
         wh = 2 * radius + outline_thickness
         sprite = sge.gfx.Sprite(None, width=wh, height=wh)
-        sprite.draw_circle(xy, xy, radius, fill, outline, outline_thickness,
-                           anti_alias)
+        sprite.draw_circle(
+            xy, xy, radius, fill=fill, outline=outline,
+            outline_thickness=outline_thickness, anti_alias=anti_alias)
+
+    cache.add(i, sprite)
+    return sprite
+
+
+def _get_polyline_sprite(points, color, thickness, anti_alias):
+    # Return a sprite for the given polygon.
+    i = ("polyline_sprite", tuple(points), tuple(color), thickness, anti_alias)
+    sprite = cache.get(i)
+    if sprite is None:
+        xlist = []
+        ylist = []
+        for point in points:
+            xlist.append(point[0])
+            ylist.append(point[1])
+        x = min(xlist)
+        y = min(ylist)
+        width = max(xlist) - x
+        height = max(ylist) - y
+
+        thickness = abs(thickness)
+        draw_x = thickness // 2
+        draw_y = thickness // 2
+        dpoints = [(a - x + draw_x, b - y + draw_y) for (a, b) in points]
+        w = width + thickness
+        h = height + thickness
+        sprite = sge.gfx.Sprite(None, width=w, height=h, origin_x=draw_x,
+                                origin_y=draw_y)
+        sprite.draw_polyline(dpoints, color, thickness, anti_alias=anti_alias)
 
     cache.add(i, sprite)
     return sprite
@@ -408,7 +509,7 @@ def _get_circle_sprite(radius, fill, outline, outline_thickness, anti_alias):
 
 def _get_polygon_sprite(points, fill, outline, outline_thickness, anti_alias):
     # Return a sprite for the given polygon.
-    i = ("poly_sprite", tuple(points),
+    i = ("polygon_sprite", tuple(points),
          tuple(fill) if fill is not None else None,
          tuple(outline) if outline is not None else None,
          outline_thickness, anti_alias)
@@ -432,8 +533,9 @@ def _get_polygon_sprite(points, fill, outline, outline_thickness, anti_alias):
         h = height + outline_thickness
         sprite = sge.gfx.Sprite(None, width=w, height=h, origin_x=draw_x,
                                 origin_y=draw_y)
-        sprite.draw_polygon(dpoints, fill, outline, outline_thickness,
-                            anti_alias)
+        sprite.draw_polygon(
+            dpoints, fill=fill, outline=outline,
+            outline_thickness=outline_thickness, anti_alias=anti_alias)
 
     cache.add(i, sprite)
     return sprite
@@ -818,7 +920,7 @@ def r_update_fade(self, complete):
         diff = (complete - self.rd["t_complete_last"]) * 2
         c = sge.gfx.Color([round(diff * 255)] * 3)
         darkener = sge.gfx.Sprite(width=w, height=h)
-        darkener.draw_rectangle(0, 0, w, h, c)
+        darkener.draw_rectangle(0, 0, w, h, fill=c)
         transition_sprite.draw_sprite(darkener, 0, 0, 0,
                                       blend_mode=sge.BLEND_RGB_SUBTRACT)
     else:
@@ -835,7 +937,7 @@ def r_update_dissolve(self, complete):
     diff = complete - self.rd["t_complete_last"]
     c = sge.gfx.Color((0, 0, 0, round(diff * 255)))
     eraser = sge.gfx.Sprite(width=w, height=h)
-    eraser.draw_rectangle(0, 0, w, h, c)
+    eraser.draw_rectangle(0, 0, w, h, fill=c)
     transition_sprite.draw_sprite(eraser, 0, 0, 0,
                                   blend_mode=sge.BLEND_RGBA_SUBTRACT)
 
@@ -865,7 +967,7 @@ def r_update_pixelate(self, complete):
         diff = (complete - self.rd["t_complete_last"]) * 5
         c = sge.gfx.Color((0, 0, 0, round(diff * 255)))
         eraser = sge.gfx.Sprite(width=w, height=h)
-        eraser.draw_rectangle(0, 0, w, h, c)
+        eraser.draw_rectangle(0, 0, w, h, fill=c)
         transition_sprite.draw_sprite(eraser, 0, 0, 0,
                                       blend_mode=sge.BLEND_RGBA_SUBTRACT)
 
@@ -1243,13 +1345,23 @@ def tg_blit(self, dest, x, y):
             sprite = get_tile(i, j)
             if sprite is not None:
                 if self.render_method == "isometric":
-                    x = sx + i * self.tile_width
-                    y = sx + j * self.tile_height / 2
-                    if i / 2 != i // 2:
-                        x += i * self.tile_width / 2
+                    x = sx + i*self.tile_width
+                    y = sx + j*self.tile_height/2
+                    if j % 2 != 0:
+                        x += self.tile_width / 2
+                elif self.render_method == "hexagonal":
+                    x = sx + i * (self.tile_width - self.meta)
+                    y = sy + j*self.tile_height
+                    if i % 2 != 0:
+                        y += self.tile_height / 2
+                elif self.render_method == "isohex":
+                    x = sx + i*self.tile_width
+                    y = sx + j * (self.tile_height - self.meta)
+                    if j % 2 != 0:
+                        x += self.tile_width / 2
                 else:
-                    x = sx + i * self.tile_width
-                    y = sy + j * self.tile_height
+                    x = sx + i*self.tile_width
+                    y = sy + j*self.tile_height
 
                 ssurf = s_set_transparency(sprite, sprite.rd["baseimages"][0])
                 dest.blit(ssurf, (int(x), int(y)))
